@@ -49,52 +49,25 @@ def unique_username_validator(form, field):
     """ Username must be unique"""
     user_manager =  current_app.user_manager
     if not user_manager.username_is_available(field.data):
-        raise ValidationError(_('This Username is no longer available. Please try another one.'))
+        raise ValidationError(_('This Username is already in use. Please try another one.'))
 
 
 def unique_email_validator(form, field):
     """ Username must be unique"""
     user_manager =  current_app.user_manager
     if not user_manager.email_is_available(field.data):
-        raise ValidationError(_('This Email is no longer available. Please try another one.'))
+        raise ValidationError(_('This Email is already in use. Please try another one.'))
 
 # ***********
 # ** Forms **
 # ***********
 
-class ChangeUsernameForm(Form):
-    new_username = StringField(_('New Username'), validators=[
-        validators.Required(_('Username is required')),
-        unique_username_validator,
-    ])
-    old_password = PasswordField(_('Old Password'), validators=[
-        validators.Required(_('Old Password is required')),
-    ])
-    next = HiddenField()
-    submit = SubmitField(_('Change Username'))
-
-    def validate(self):
-        user_manager =  current_app.user_manager
-
-        # Add custom username validator if needed
-        has_been_added = False
-        for v in self.new_username.validators:
-            if v==user_manager.username_validator:
-                has_been_added = True
-        if not has_been_added:
-            self.new_username.validators.append(user_manager.username_validator)
-
-        # Validate field-validators
-        if not super(ChangeUsernameForm, self).validate():
-            return False
-
-        # Verify current_user and current_password
-        if not current_user or not user_manager.verify_password(self.old_password.data, current_user.password):
-            self.old_password.errors.append(_('Old Password is incorrect'))
-            return False
-
-        # All is well
-        return True
+class AddEmailForm(Form):
+    email = StringField(_('Email'), validators=[
+        validators.Required(_('Email is required')),
+        validators.Email(_('Invalid Email')),
+        unique_email_validator])
+    submit = SubmitField(_('Add Email'))
 
 class ChangePasswordForm(Form):
     old_password = PasswordField(_('Old Password'), validators=[
@@ -135,6 +108,40 @@ class ChangePasswordForm(Form):
         # All is well
         return True
 
+class ChangeUsernameForm(Form):
+    new_username = StringField(_('New Username'), validators=[
+        validators.Required(_('Username is required')),
+        unique_username_validator,
+    ])
+    old_password = PasswordField(_('Old Password'), validators=[
+        validators.Required(_('Old Password is required')),
+    ])
+    next = HiddenField()
+    submit = SubmitField(_('Change Username'))
+
+    def validate(self):
+        user_manager =  current_app.user_manager
+
+        # Add custom username validator if needed
+        has_been_added = False
+        for v in self.new_username.validators:
+            if v==user_manager.username_validator:
+                has_been_added = True
+        if not has_been_added:
+            self.new_username.validators.append(user_manager.username_validator)
+
+        # Validate field-validators
+        if not super(ChangeUsernameForm, self).validate():
+            return False
+
+        # Verify current_user and current_password
+        if not current_user or not user_manager.verify_password(self.old_password.data, current_user.password):
+            self.old_password.errors.append(_('Old Password is incorrect'))
+            return False
+
+        # All is well
+        return True
+
 class ForgotPasswordForm(Form):
     email = StringField(_('Email'), validators=[
         validators.Required(_('Email is required')),
@@ -176,14 +183,20 @@ class LoginForm(Form):
         if not super(LoginForm, self).validate():
             return False
 
+        # Find user by username and/or email
+        user = None
+        user_email = None
         if user_manager.enable_username:
-            # Find user by username or email address
+            # Find user by username
             user = user_manager.find_user_by_username(self.username.data)
+
+            # Find user by email address (username field)
             if not user and user_manager.enable_email:
-                user = user_manager.find_user_by_email(self.username.data)
+                user, user_email = user_manager.find_user_by_email(self.username.data)
+
         else:
-            # Find user by email address
-            user = user_manager.find_user_by_email(self.email.data)
+            # Find user by email address (email field)
+            user, user_email = user_manager.find_user_by_email(self.email.data)
 
         # Validate user and password
         if not user or not user_manager.verify_password(self.password.data, user.password):
