@@ -39,9 +39,11 @@ def set_default_settings(user_manager, app_config):
     um.change_password_url      = sd('USER_CHANGE_PASSWORD_URL',        '/user/change-password')
     um.change_username_url      = sd('USER_CHANGE_USERNAME_URL',        '/user/change-username')
     um.confirm_email_url        = sd('USER_CONFIRM_EMAIL_URL',          '/user/confirm-email/<token>')
+    um.email_action_url         = sd('USER_EMAIL_ACTION_URL',           '/user/email/<id>/<action>')
     um.forgot_password_url      = sd('USER_FORGOT_PASSWORD_URL',        '/user/forgot-password')
     um.login_url                = sd('USER_LOGIN_URL',                  '/user/sign-in')
     um.logout_url               = sd('USER_LOGOUT_URL',                 '/user/sign-out')
+    um.manage_emails_url        = sd('USER_MANAGE_EMAILS_URL',          '/user/manage-emails')
     um.register_url             = sd('USER_REGISTER_URL',               '/user/register')
     um.resend_confirm_email_url = sd('USER_RESEND_CONFIRM_EMAIL_URL',   '/user/resend-confirm-email')
     um.reset_password_url       = sd('USER_RESET_PASSWORD_URL',         '/user/reset-password/<token>')
@@ -52,6 +54,7 @@ def set_default_settings(user_manager, app_config):
     um.change_username_template = sd('USER_CHANGE_USERNAME_TEMPLATE',  'flask_user/change_username.html')
     um.forgot_password_template = sd('USER_FORGOT_PASSWORD_TEMPLATE',  'flask_user/forgot_password.html')
     um.login_template           = sd('USER_LOGIN_TEMPLATE',            'flask_user/login.html')
+    um.manage_emails_template   = sd('USER_MANAGE_EMAILS_TEMPLATE',    'flask_user/manage_emails.html')
     um.register_template        = sd('USER_REGISTER_TEMPLATE',         'flask_user/register.html')
     um.resend_confirm_email_template = sd('USER_RESEND_CONFIRM_EMAIL_TEMPLATE', 'flask_user/resend_confirm_email.html')
     um.reset_password_template       = sd('USER_RESET_PASSWORD_TEMPLATE',       'flask_user/reset_password.html')
